@@ -32,9 +32,9 @@ def main(args, spec: str, spec_file: str):
     async def run():
         servers = await run_spec(_spec, *args)
         try:
-            await asyncio.gather(*[w.finished() for w in servers.values()])
+            await asyncio.gather(*(w.finished() for w in servers.values()))
         except KeyboardInterrupt:
-            await asyncio.gather(*[w.close() for w in servers.values()])
+            await asyncio.gather(*(w.close() for w in servers.values()))
 
     asyncio.get_event_loop().run_until_complete(run())
 
