@@ -1,31 +1,27 @@
-from array import array
-from functools import partial
-import traceback
-import importlib
-from enum import Enum
+from __future__ import annotations
 
-import dask
-from dask.base import normalize_token
+import importlib
+import traceback
+from array import array
+from enum import Enum
+from functools import partial
+from types import ModuleType
 
 import msgpack
 
+import dask
+from dask.base import normalize_token
+from dask.utils import typename
+
+from ..utils import ensure_bytes, has_keyword
 from . import pickle
-from ..utils import has_keyword, typename, ensure_bytes
-from .compression import maybe_compress, decompress
-from .utils import (
-    unpack_frames,
-    pack_frames_prelude,
-    frame_split_size,
-    msgpack_opts,
-)
-
-
-lazy_registrations = {}
+from .compression import decompress, maybe_compress
+from .utils import frame_split_size, msgpack_opts, pack_frames_prelude, unpack_frames
 
 dask_serialize = dask.utils.Dispatch("dask_serialize")
 dask_deserialize = dask.utils.Dispatch("dask_deserialize")
 
-_cached_allowed_modules = {}
+_cached_allowed_modules: dict[str, ModuleType] = {}
 
 
 def dask_dumps(x, context=None):
@@ -69,15 +65,26 @@ def pickle_dumps(x, context=None):
 
 def pickle_loads(header, frames):
     x, buffers = frames[0], frames[1:]
-    writeable = header["writeable"]
-    for i in range(len(buffers)):
-        mv = memoryview(buffers[i])
-        if writeable[i] == mv.readonly:
+
+    writeable = header.get("writeable")
+    if not writeable:
+        writeable = len(buffers) * (None,)
+
+    new = []
+    memoryviews = map(memoryview, buffers)
+    for w, mv in zip(writeable, memoryviews):
+        if w == mv.readonly:
             if mv.readonly:
-                buffers[i] = memoryview(bytearray(mv)).cast(mv.format, mv.shape)
+                mv = memoryview(bytearray(mv))
             else:
-                buffers[i] = memoryview(bytes(mv)).cast(mv.format, mv.shape)
-    return pickle.loads(x, buffers=buffers)
+                mv = memoryview(bytes(mv))
+            if mv.nbytes > 0:
+                mv = mv.cast(mv.format, mv.shape)
+            else:
+                mv = mv.cast(mv.format)
+        new.append(mv)
+
+    return pickle.loads(x, buffers=new)
 
 
 def import_allowed_module(name):
@@ -191,7 +198,9 @@ def check_dask_serializable(x):
     return False
 
 
-def serialize(x, serializers=None, on_error="message", context=None):
+def serialize(
+    x, serializers=None, on_error="message", context=None, iterate_collection=None
+):
     r"""
     Convert object to a header and list of bytestrings
 
@@ -205,6 +214,12 @@ def serialize(x, serializers=None, on_error="message", context=None):
     per-class serialization methods. ``None`` gives the default list
     ``['dask', 'pickle']``.
 
+    Notes on the ``iterate_collection`` argument (only relevant when
+    ``x`` is a collection):
+    - ``iterate_collection=True``: Serialize collection elements separately.
+    - ``iterate_collection=False``: Serialize collection elements together.
+    - ``iterate_collection=None`` (default): Infer the best setting.
+
     Examples
     --------
     >>> serialize(1)
@@ -230,11 +245,20 @@ def serialize(x, serializers=None, on_error="message", context=None):
     if serializers is None:
         serializers = ("dask", "pickle")  # TODO: get from configuration
 
+    # Handle obects that are marked as `Serialize`, or that are
+    # already `Serialized` objects (don't want to serialize them twice)
     if isinstance(x, Serialized):
         return x.header, x.frames
+    if isinstance(x, Serialize):
+        return serialize(
+            x.data,
+            serializers=serializers,
+            on_error=on_error,
+            context=context,
+            iterate_collection=True,
+        )
 
-    if type(x) in (list, set, tuple, dict):
-        iterate_collection = False
+    if iterate_collection is None and type(x) in (list, set, tuple, dict):
         if type(x) is list and "msgpack" in serializers:
             # Note: "msgpack" will always convert lists to tuples
             #       (see GitHub #3716), so we should iterate
@@ -302,14 +326,14 @@ def serialize(x, serializers=None, on_error="message", context=None):
     tb = ""
 
     for name in serializers:
-        dumps, loads, wants_context = families[name]
+        dumps, _, wants_context = families[name]
         try:
             header, frames = dumps(x, context=context) if wants_context else dumps(x)
             header["serializer"] = name
             return header, frames
         except NotImplementedError:
             continue
-        except Exception as e:
+        except Exception:
             tb = traceback.format_exc()
             break
 
@@ -334,7 +358,7 @@ def deserialize(header, frames, deserializers=None):
     ----------
     header : dict
     frames : list of bytes
-    deserializers : Optional[Dict[str, Tuple[Callable, Callable, bool]]]
+    deserializers : dict[str, tuple[Callable, Callable, bool]] | None
         An optional dict mapping a name to a (de)serializer.
         See `dask_serialize` and `dask_deserialize` for more.
 
@@ -384,7 +408,9 @@ def deserialize(header, frames, deserializers=None):
     return loads(header, frames)
 
 
-def serialize_and_split(x, serializers=None, on_error="message", context=None):
+def serialize_and_split(
+    x, serializers=None, on_error="message", context=None, size=None
+):
     """Serialize and split compressable frames
 
     This function is a drop-in replacement of `serialize()` that calls `serialize()`
@@ -406,7 +432,7 @@ def serialize_and_split(x, serializers=None, on_error="message", context=None):
         frames, header.get("compression") or [None] * len(frames)
     ):
         if compression is None:  # default behavior
-            sub_frames = frame_split_size(frame)
+            sub_frames = frame_split_size(frame, n=size)
             num_sub_frames.append(len(sub_frames))
             offsets.append(len(out_frames))
             out_frames.extend(sub_frames)
@@ -507,70 +533,6 @@ class Serialized:
         return not (self == other)
 
 
-def extract_serialize(x) -> tuple:
-    """Pull out Serialize objects from message
-
-    This also remove large bytestrings from the message into a second
-    dictionary.
-
-    Examples
-    --------
-    >>> from distributed.protocol import to_serialize
-    >>> msg = {'op': 'update', 'data': to_serialize(123)}
-    >>> extract_serialize(msg)
-    ({'op': 'update'}, {('data',): <Serialize: 123>}, set())
-    """
-    typ_x: type = type(x)
-    if typ_x is dict:
-        x_d: dict = x
-        x_items = x_d.items()
-        x2 = {}
-    elif typ_x is list:
-        x_l: list = x
-        x_items = enumerate(x_l)
-        x2 = len(x_l) * [None]
-
-    ser = {}
-    bytestrings = set()
-    path = ()
-    _extract_serialize(x_items, x2, ser, bytestrings, path)
-    return x2, ser, bytestrings
-
-
-def _extract_serialize(x_items, x2, ser: dict, bytestrings: set, path: tuple) -> None:
-    for k, v in x_items:
-        path_k = path + (k,)
-        typ_v: type = type(v)
-        if typ_v is dict:
-            v_d: dict = v
-            v_items = v_d.items()
-            x2[k] = v2 = {}
-            _extract_serialize(v_items, v2, ser, bytestrings, path_k)
-        elif typ_v is list:
-            v_l: list = v
-            v_items = enumerate(v_l)
-            x2[k] = v2 = len(v_l) * [None]
-            _extract_serialize(v_items, v2, ser, bytestrings, path_k)
-        elif typ_v is Serialize or typ_v is Serialized:
-            ser[path_k] = v
-        elif typ_v is bytes:
-            v_b: bytes = v
-            if len(v_b) > 2 ** 16:
-                ser[path_k] = to_serialize(v_b)
-                bytestrings.add(path_k)
-            else:
-                x2[k] = v_b
-        elif typ_v is bytearray:
-            v_ba: bytearray = v
-            if len(v_ba) > 2 ** 16:
-                ser[path_k] = to_serialize(v_ba)
-                bytestrings.add(path_k)
-            else:
-                x2[k] = v_ba
-        else:
-            x2[k] = v
-
-
 def nested_deserialize(x):
     """
     Replace all Serialize and Serialized values nested in *x*
